@@ -48,7 +48,7 @@ class _LazyMangledNamesMap {
 
   _LazyMangledNamesMap(this._jsMangledNames);
 
-  String operator [](String key) {
+  String? operator [](String key) {
     var result = JS('var', '#[#]', _jsMangledNames, key);
     // Filter out all non-string values to protect against polution from
     // ancillary fields in [_jsMangledNames].
@@ -65,8 +65,8 @@ class _LazyMangledNamesMap {
 class _LazyMangledInstanceNamesMap extends _LazyMangledNamesMap {
   _LazyMangledInstanceNamesMap(_jsMangledNames) : super(_jsMangledNames);
 
-  String operator [](String key) {
-    String result = super[key];
+  String? operator [](String key) {
+    String? result = super[key];
     String setterPrefix = JS_GET_NAME(JsGetName.SETTER_PREFIX);
     if (result == null && key.startsWith(setterPrefix)) {
       String getterPrefix = JS_GET_NAME(JsGetName.GETTER_PREFIX);
@@ -82,7 +82,7 @@ class _LazyMangledInstanceNamesMap extends _LazyMangledNamesMap {
 }
 
 /// Implements the inverse of [_LazyMangledNamesMap]. As it would be too
-/// expensive to seach the mangled names map for a value that corresponds to
+/// expensive to search the mangled names map for a value that corresponds to
 /// the lookup key on each invocation, we compute the full mapping in demand
 /// and cache it. The cache is invalidated when the underlying [_jsMangledNames]
 /// object changes its length. This condition is sufficient as the name mapping
@@ -94,7 +94,7 @@ class _LazyReflectiveNamesMap {
   final _jsMangledNames;
   final bool _isInstance;
   int _cacheLength = 0;
-  Map<String, String> _cache;
+  Map<String, String>? _cache;
 
   _LazyReflectiveNamesMap(this._jsMangledNames, this._isInstance);
 
@@ -124,12 +124,12 @@ class _LazyReflectiveNamesMap {
   int get _jsMangledNamesLength =>
       JS('int', 'Object.keys(#).length', _jsMangledNames);
 
-  String operator [](String key) {
+  String? operator [](String key) {
     if (_cache == null || _jsMangledNamesLength != _cacheLength) {
       _cache = _updateReflectiveNames();
       _cacheLength = _jsMangledNamesLength;
     }
-    return _cache[key];
+    return _cache![key];
   }
 }
 
@@ -148,9 +148,9 @@ List extractKeys(victim) {
 /// This is used, for example, to return unmangled names from TypeImpl.toString
 /// *if* names are being preserved for other reasons (use of dart:mirrors, for
 /// example).
-String unmangleGlobalNameIfPreservedAnyways(String name) {
-  var names = JS_EMBEDDED_GLOBAL('=Object', MANGLED_GLOBAL_NAMES);
-  return JsCache.fetch(names, name);
+String? unmangleGlobalNameIfPreservedAnyways(String name) {
+  var names = JS_EMBEDDED_GLOBAL('', MANGLED_GLOBAL_NAMES);
+  return JS('String|Null', '#', JsCache.fetch(names, name));
 }
 
 String unmangleAllIdentifiersIfPreservedAnyways(String str) {
